@@ -12,7 +12,8 @@ from django.forms import Form
 class ReportMetaClass(type):
     def __new__(meta, classname, bases, classDict):
         cls = type.__new__(meta, classname, bases, classDict)
-        if not classname in ["Report", "HTSQLReport"]:
+        abstract = hasattr(cls, "__abtract__") and cls.__abstract__
+        if not cls.abstract():
             if not "name" in classDict:
                 raise AttributeError("%s class should have a name" % classname)
         return cls
@@ -21,6 +22,7 @@ class ReportMetaClass(type):
 class Report(object):
 
     __metaclass__ = ReportMetaClass
+    __abstract__ = True
 
     NO_RUN, RUNNING, DONE, FAILED = 0, 1, 2, -1
     status_description = {0: "No run",
@@ -41,6 +43,10 @@ class Report(object):
         self._status = None
         self.parameters = parameters or {}
 
+    @classmethod
+    def abstract(cls):
+        return getattr(cls, "__abstract__", False)
+
     @classmethod
     def get_verbose_name(self):
         return getattr(self, "verbose_name", self.name)
@@ -76,17 +82,25 @@ class Report(object):
         self.report_handler.post_run(report=self, **kwargs)
         return self.status()
 
-    def write_line(self, line):
-        self.formatter.write([field.encode(getattr(self, "encoding", "utf8")) if hasattr(field, "encode") else field for field in line])
-
     def get_data(self):
+        header = self.get_header()
+        if header:
+            self.write_header(header)
         for line in self._data:
             self.write_line(line)
         self.content = self.output.getvalue()
         return self.content
 
     def post_populate(self):
-        raise NotImplemented()
+        now_ = datetime.datetime.now().strftime("%Y%M%d_%H%m%s")
+        self.filename = "/tmp/%s_%s_%d" % (self.name, now_,  id(self))
+        self.as_file(self.filename)
+
+    def write_line(self, line):
+        raise NotImplementedError()
+
+    def write_header(self, header):
+        raise NotImplementedError()
 
     def as_file(self, filename):
         file_ = open(filename, "wb")
@@ -95,6 +109,15 @@ class Report(object):
         file_.write(self.content)
         return file_
 
+    def write_data(self, data):
+        self.formatter.write([field.encode(getattr(self, "encoding", "utf8")) if hasattr(field, "encode") else field for field in data])
+
+    def write_line(self, line):
+        self.write_data(line)
+
+    def get_header(self):
+        return []
+
     @classmethod
     def get_form_class(cls):
         return Form
@@ -112,6 +135,7 @@ def async_populate(instance):
         
 
 class HTSQLReport(Report):
+    __abstract__ = True
     def __init__(self, *args, **kwargs):
         super(HTSQLReport, self).__init__(*args, **kwargs)
         self._connexion = getattr(self, "connexion", None) or getattr(settings,"HTSQL_REPORT_CONNEXION")
@@ -120,7 +144,21 @@ class HTSQLReport(Report):
     def populate(self):
         self._data = self._session.produce(self.query, **self.parameters)
 
-    def post_populate(self):
-        now_ = datetime.datetime.now().strftime("%Y%M%d_%H%m%s")
-        self.filename = "/tmp/%s_%s_%d" % (self.name, now_,  id(self))
-        self.as_file(self.filename)
+class DjangoReport(Report):
+    __abstract__ = True
+
+    def populate(self):
+        self._data = self.queryset
+
+    def write_header(self, header):
+        self.write_data(header)
+
+    def write_line(self, line):
+        row = self.get_row(line)
+        super(DjangoReport, self).write_line(row)
+
+    def get_row(self, line):
+        raise NotImplementedError()
+
+    def get_header(self):
+        return getattr(self, "header", [])
